package com.extrieve.quickcapture;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.widget.Toast;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.extrieve.quickcapture.sdk.*;

import java.io.IOException;

/** QuickcapturePlugin */
public class QuickcapturePlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity

  private int REQUEST_CODE_PERMISSIONS = 1001;
  public int REQUEST_CODE_FILE_RETURN = 1002;
  private MethodChannel channel;
  private Activity activity;
  private Context context;
  private ImgHelper ImageHelper;


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "quickcapture");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();

    try {
      ImageHelper = new ImgHelper(context);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    try {
      ImageHelper.Init();
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    Config.CaptureSupport.OutputPath = BuildStoragePath();
    if (call.method.equals("getPlatformVersion")) {
      OpenCameraActivity();
      result.success(Config.CaptureSupport.OutputPath);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {
    activity = null;
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  private void OpenCameraActivity() {
    //before starting camera - configuration can set
    //String quality = ImageHelper.getCurrentImageQuality();
    try {
      //moving to camera activity in library
      Intent CameraIntent = new Intent(activity, Class.forName("com.extrieve.quickcapture.sdk.CameraHelper"));
      //photoURI = CameraSupport.CamConfigClass.OutputPath;
      Uri photoURI = Uri.parse(Config.CaptureSupport.OutputPath);
      activity.grantUriPermission(activity.getPackageName(), photoURI,
              Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
      if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP) {
        CameraIntent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
      }
      activity.startActivityForResult(CameraIntent, REQUEST_CODE_FILE_RETURN);

    } catch (ClassNotFoundException e) {
      Toast.makeText(activity, "Failed to open camera + ", Toast.LENGTH_LONG).show();
      e.printStackTrace();
    }
  }

  @NonNull
  private String BuildStoragePath() {
    //File direct = new File(Environment.getExternalStorageDirectory(), ".GoNoGoImages");
    //return direct.getAbsolutePath();
    ContextWrapper c = new ContextWrapper(activity);
    String path = c.getExternalFilesDir(".GoNoGoImages").getAbsolutePath();
    return path;
  }
}
